@@ -30,6 +30,8 @@ import uuid
 from PIL import Image
 from majormode.mercurius.constant.place import SearchIntent
 from majormode.mercurius.model.place import AddressComponentName
+from majormode.perseus.constant.area import AREA_LEVEL_COUNTRY
+from majormode.perseus.constant.area import AREA_LEVEL_DISTRICT
 from majormode.perseus.constant.obj import ObjectStatus
 from majormode.perseus.constant.privacy import Visibility
 from majormode.perseus.constant.team import MemberRole
@@ -255,15 +257,22 @@ class PlaceService(BaseRdbmsService):
         return file_path_name
 
     @staticmethod
-    def __find_country_with_ip_address(ip_address, locale=None):
+    def __find_area_with_ip_address(
+            ip_address,
+            area_level=None,
+            locale=None):
         """
-        Find a country with an IP address
+        Find an administrative subdivision with an IP address
 
 
         :param ip_address:  IPv4 address of the machine of a user.  It
             corresponds to a tuple consisting of four decimal numbers, each
             ranging from `0` to `255`.
 
+        :param area_level: The level of the administrative subdivision to
+            return a list of schools.  By default, it corresponds to a country
+            level.  The minimal area level is `AREA_LEVEL_DISTRICT`.
+
         :param locale: An object `Locale` specifying the language to return
             the name of the country.
 
@@ -271,17 +280,22 @@ class PlaceService(BaseRdbmsService):
         :return: An object representing the information of the country found
             with this IP address.
         """
+        if area_level is None:
+            area_level = AREA_LEVEL_COUNTRY
+
+        # Search from the smallest possible administrative subdivision to
+        # improve a lot the performance (milliseconds v.s. a few seconds).
         areas = AreaService().find_areas_with_ip_address(
             ip_address,
-            lowest_area_level=4,  # @note: To get faster response time
-            highest_area_level=0,
+            lowest_area_level=min(AREA_LEVEL_DISTRICT, area_level),
+            highest_area_level=area_level,
             locale=locale)
 
         # Filter the country area.
         areas = [
             area
             for area in areas
-            if area.area_level == 0
+            if area.area_level == area_level
         ]
 
         area = None if len(areas) == 0 else areas[0]
@@ -1586,9 +1600,14 @@ class PlaceService(BaseRdbmsService):
                 """
                 SELECT
                     place_id
-                  FROM
+                  FROM ONLY
                     place
-                """
+                  WHERE 
+                    object_status <> %(OBJECT_STATUS_DELETED)s
+                """,
+                {
+                    'OBJECT_STATUS_DELETED': ObjectStatus.deleted,
+                }
             )
 
             place_ids = [
@@ -1603,6 +1622,7 @@ class PlaceService(BaseRdbmsService):
             self,
             account_id: uuid.UUID = None,
             area_id: uuid.UUID = None,
+            area_level: int = None,
             bounding_box: BoundingBox = None,
             categories=None,
             client_ip_address: Tuple[int, int, int,int] = None,
@@ -1627,6 +1647,11 @@ class PlaceService(BaseRdbmsService):
         :param area_id: The identification of an administrative subdivision to
             search for place.
 
+        :param area_level: The level of the administrative subdivision to
+            return a list of schools.  By default, it corresponds to a country
+            level.  This argument is used when the argument `location` is
+            passed to this function.
+
         :param bounding_box: An object `BoundingBox` that represent the
             rectangle area to search for places.
 
@@ -1737,10 +1762,14 @@ class PlaceService(BaseRdbmsService):
         #
         if bounding_box:
             geometry_wkt = self.__build_bounding_box_wkt(bounding_box)
-        elif client_ip_address:
-            country = self.__find_country_with_ip_address(client_ip_address, locale=locale)
-            if country is not None:
-                area_id = country.area_id
+        elif not area_id and client_ip_address:
+            area = self.__find_area_with_ip_address(
+                client_ip_address,
+                area_level=area_level,
+                locale=locale)
+
+            if area is not None:
+                area_id = area.area_id
 
 #         location_provider_count = sum([provider is not None for provider in [ area_id, location, bounds, ip_address]])
 #         if keywords is None and location_provider_count != 1:
